/*
Copyright (c) 2009, Intel Corporation
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "atcommand_parser.h"
#include "modem_channel.h"
#include "dund_log.h"

int default_command_handler(char* command);
int ato_command_handler(char* command);
int at_command_handler(char* command);
int ate_command_handler(char* command);

AT_Command AT_COMMAND_TABLE[] = {
{"", NULL, at_command_handler},
{" ", NULL, at_command_handler},
{"&C", NULL, default_command_handler},
{"&D", NULL, default_command_handler},
{"&F", NULL, default_command_handler},
{"+CBC", NULL, default_command_handler},
{"+CBST", NULL, default_command_handler},
{"+CCFC", NULL, default_command_handler},
{"+CCVI", NULL, default_command_handler},
{"+CEER", NULL, default_command_handler},
{"+CFUN", NULL, default_command_handler},
{"+CGACT", NULL, default_command_handler},
{"+CGATT", NULL, default_command_handler},
{"+CGDATA", NULL, default_command_handler},
{"+CGDCONT", NULL, default_command_handler},
{"+CGEQMIN", NULL, default_command_handler},
{"+CGEQNEG", NULL, default_command_handler},
{"+CGEQREQ", NULL, default_command_handler},
{"+CGEREP", NULL, default_command_handler},
{"+CGMI", NULL, default_command_handler},
{"+CGMM", NULL, default_command_handler},
{"+CGMN", NULL, default_command_handler},
{"+CMGR", NULL, default_command_handler},
{"+CGPADDR", NULL, default_command_handler},
{"+CGQMIN", NULL, default_command_handler},
{"+CGQREQ", NULL, default_command_handler},
{"+CGREG", NULL, default_command_handler},
{"+CGSN", NULL, default_command_handler},
{"+CHUP", NULL, default_command_handler},
{"+CIMI", NULL, default_command_handler},
{"+CLAC", NULL, default_command_handler},
{"+CLCC", NULL, default_command_handler},
{"+CLCK", NULL, default_command_handler},
{"+CLIP", NULL, default_command_handler},
{"+CLIR", NULL, default_command_handler},
{"+CMAR", NULL, default_command_handler},
{"+CMEE", NULL, default_command_handler},
{"+CMFG", NULL, default_command_handler},
{"+CMGD", NULL, default_command_handler},
{"+CMGL", NULL, default_command_handler},
{"+CMGR", NULL, default_command_handler},
{"+CMGS", NULL, default_command_handler},
{"+CMOD", NULL, default_command_handler},
{"+CNMI", NULL, default_command_handler},
{"+CNUM", NULL, default_command_handler},
{"+COPN", NULL, default_command_handler},
{"+COPS", NULL, default_command_handler},
{"+CPBR", NULL, default_command_handler},
{"+CPBS", NULL, default_command_handler},
{"+CPAS", NULL, default_command_handler},
{"+CPIN", NULL, default_command_handler},
{"+CPBR", NULL, default_command_handler},
{"+CPBW", NULL, default_command_handler},
{"+CPMS", NULL, default_command_handler},
{"+CPWD", NULL, default_command_handler},
{"+CREG", NULL, default_command_handler},
{"+CRFREP", NULL, default_command_handler},
{"+CRLP", NULL, default_command_handler},
{"+CSCA", NULL, default_command_handler},
{"+CSCS", NULL, default_command_handler},
{"+CSNS", NULL, default_command_handler},
{"+CSN", NULL, default_command_handler},
{"+CSSN", NULL, default_command_handler},
{"+CSTA", NULL, default_command_handler},
{"+CUSD", NULL, default_command_handler},
{"+CV117", NULL, default_command_handler},
{"+CVHU", NULL, default_command_handler},
{"+GMI", NULL, default_command_handler},
{"+GMM", NULL, default_command_handler},
{"+GMR", NULL, default_command_handler},
{"+GSN", NULL, default_command_handler},
{"+ICF", NULL, default_command_handler},
{"+IFC", NULL, default_command_handler},
{"+ILRR", NULL, default_command_handler},
{"+IPR", NULL, default_command_handler},
{"+VTS", NULL, default_command_handler},
{"+WS46", NULL, default_command_handler},
{"A", NULL, default_command_handler},
{"D", NULL, default_command_handler},
{"E", NULL, ate_command_handler},
{"H", NULL, default_command_handler},
{"I", NULL, default_command_handler},
{"O", NULL, ato_command_handler},
{"P", NULL, default_command_handler},
{"Q", NULL, default_command_handler},
{"D", NULL, default_command_handler},
{"Z", NULL, default_command_handler},
{"X", NULL, default_command_handler},
{"V", NULL, default_command_handler},
{"T", NULL, default_command_handler},
{"S10", NULL, default_command_handler},
{"S0", NULL, default_command_handler},
{"S3", NULL, default_command_handler},
{"S4", NULL, default_command_handler},
{"S5", NULL, default_command_handler},
{"S6", NULL, default_command_handler},
{"S7", NULL, default_command_handler},
{"S8", NULL, default_command_handler},
{"S9", NULL, default_command_handler}
};

int default_command_handler(char* command){
    /*default support*/
    char buffer[1024];
    dund_log("command %s supported\n", command);

    buffer[0] = 'A';
    buffer[1] = 'T';
    strcpy(buffer + 2, command);
    flush_buffer_to_modem(buffer, strlen(command) + 2);
    flush_buffer_to_modem("\r\n", strlen("\r\n"));
    return 0;
}

int at_command_handler(char* command) {
    //only take care about ATE
    char buffer[1024];
    char* pate_string = NULL;
    if (pate_string=strcasestr(command, " E")) {
        dund_log("%s\n", pate_string);
        at_command_echo = pate_string[2] - '0';
        pate_string[2] = '0';
        dund_log("echo at command:%d\n", at_command_echo);
    }
    buffer[0] = 'A';
    buffer[1] = 'T';
    strcpy(buffer + 2, command);
    flush_buffer_to_modem(buffer, strlen(command) + 2);
    flush_buffer_to_modem("\r\n", strlen("\r\n"));
}

int ato_command_handler(char* command){
    char buffer[1024];
    int length = strlen(command);
    dund_log("command %s supported\n", command);

    switch (length) {
      case 1:
      //ATO
      current_modem_mode = DATA_MODE;break;
      case 2:
      //ATO0?
      if (command[1] == '0') {current_modem_mode = DATA_MODE;}break;
    }
        
    buffer[0] = 'A';
    buffer[1] = 'T';
    strcpy(buffer + 2, command);
    flush_buffer_to_modem(buffer, strlen(command) + 2);
    flush_buffer_to_modem("\r\n", strlen("\r\n"));
    return 0;
}

int ate_command_handler(char* command){
    char buffer[1024];
    int length = strlen(command);
    dund_log("command %s supported\n", command);

    //only support ATE1 or ATE0
    if (length < strlen("E1")) {
       return 0;
    }
    switch (command[1]) {
        case '1':
              at_command_echo = 1; break;
        case '0':
              at_command_echo = 0; break;
        default:
              dund_log("Not supported\n");
    }

    buffer[0] = 'A';
    buffer[1] = 'T';
    strcpy(buffer + 2, command);
// should not enable AT command echo in Modem
    return 0;
}

