/*
Copyright (c) 2009, Intel Corporation
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "atcommand_parser.h"
#include "local_channel.h"

int split_into_commands(char* p_command_line, AT_CommandArray* patcommand_array)
{
    int i = 0;
    int length = 0;
    int cur_pos = 0;
    int last_command_pos = 0;
    char * pline = strdup(p_command_line);
    
    length = strlen(pline);
    char** command_array;

    for (i = length -1; i > 0; i --) {
        if (!((pline[i] == '\r') || (pline[i] == '\n') || (pline[i] == ' ')))
            break;
        pline[i] = '\0';
    }

    for (i = 0 ; i < length; i ++) {
        if (!((pline[i] == '\r') || (pline[i] == '\n') || (pline[i] == ' ')))
            break;
        pline[i] = '\0';
    }

    if (!((pline[i] == 'a' || pline[i++] == 'A') && 
        (pline[i] == 't' || pline[i++] == 'T'))){
        free (pline);
        // Not a AT command
        return 0;
    }

    command_array = malloc(length*sizeof(char*));
    
    memset(command_array, 0, length * sizeof (char*));
    
    length = strlen(pline);
    
    last_command_pos = i;
    for (; i < strlen(pline); i ++){
        if (pline[i] == ';'&& pline[last_command_pos] != 'D'){
            pline[i] = '\0';
            command_array[cur_pos++] = strdup(&pline[last_command_pos]);
            last_command_pos = i+1;
        }
    }
    
    //last command
    command_array[cur_pos++] = strdup(&pline[last_command_pos]);
    patcommand_array->command_array = command_array;
    patcommand_array->number = cur_pos;
    
    free (pline);
    return cur_pos;
}

int parser_at_command(char* at_command){
    int i = 0;
    int ret_value = 0;
    
    char* pCommand = strdup(at_command);
    
    for (i = 0; i < strlen(pCommand); i ++){
        pCommand[i] = toupper(pCommand[i]);
    }
    
    for (i = 0; i < SUPPORTED_AT_COMMAND_NUMBER; i ++){
        if (!strncmp(AT_COMMAND_TABLE[i].command, pCommand, strlen(AT_COMMAND_TABLE[i].command))){
            //? should we decide filter or not here?
            ret_value = AT_COMMAND_TABLE[i].processor(at_command);
            break;
        }
    }

    if (i == SUPPORTED_AT_COMMAND_NUMBER){
        write_to_local_channel("+CME ERROR:4\r\n", strlen("+CME ERROR:4\r\n"));
    }
    
    free(pCommand);
    
    return ret_value;
}

int process_line(char* line){
    AT_CommandArray atcommand_array;
    int i;
    int number_command = 0;
    char* output_line = NULL;
    
    number_command = split_into_commands(line, &atcommand_array);
    
    if (number_command == 0){
        //Not a AT command, return a error number
        dund_log("AT Parser: not a at command :%s\n", line);
        flush_buffer_to_modem(line, strlen(line));
        flush_buffer_to_modem("\r\n", strlen("\r\n"));
        return 0;
    }
    
    for (i = 0; i < number_command; i ++){
        parser_at_command(atcommand_array.command_array[i]);
    }
    
    free(atcommand_array.command_array);
    
    return 0;
}

