/*
Copyright (c) 2009, Intel Corporation
All rights reserved.
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <fcntl.h>
#include <pthread.h>
#include "local_channel.h"
#include "modem_channel.h"

#define LOCAL_CHANNEL_PATH "/tmp/at_filter_proxy"
#define MODEM_NODE "/dev/ttyHS1"

static pthread_t lchannel_tid, modem_tid;
int initialize_filter(char* local_channel, char* modem_dev){
    int ret_value;
    ret_value = open_local_channel(local_channel);

    if (ret_value < 0) return ret_value;

    ret_value = open_modem_channel(modem_dev, O_RDWR);

    if (ret_value < 0) return ret_value;

    pthread_create(&lchannel_tid, NULL, local_channel_reader, NULL);

    pthread_create(&modem_tid, NULL, modem_reader, NULL);
    return 0;
}

int shutdown_channels(){
    close_modem_channel();
    close_local_channel();
}

int waitfor_filter_socket_close(){
    pthread_join(lchannel_tid, NULL);

    pthread_join(modem_tid, NULL);
    return 0;
}

int finalize_filter(){
    close_modem_channel();
    close_local_channel();

    pthread_join(lchannel_tid, NULL);
    pthread_join(modem_tid, NULL);
    return 0;
}
