/*
Copyright (c) 2009, Intel Corporation
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#include "local_channel.h"

extern int errno;
static int local_socket_channel = -1;

int open_local_channel(unsigned char* path){
    struct sockaddr_un local_address, remote_address;
    size_t local_size, remote_size = sizeof(struct sockaddr_un);
    int listen_socket;

    listen_socket = socket(AF_UNIX, SOCK_STREAM, 0);

    local_address.sun_family = AF_UNIX;  
    strcpy(local_address.sun_path, path);
    unlink(local_address.sun_path);

    local_size = strlen(local_address.sun_path) + sizeof(local_address.sun_family);
    bind(listen_socket, (struct sockaddr *)&local_address, local_size);
 
    if (listen(listen_socket, 5) == -1) {
        dund_log("Failed in listen local socket:%s\n", strerror(errno));
    }

    local_socket_channel = accept(listen_socket, (struct sockaddr *)&remote_address, &remote_size); 

    close(listen_socket);
    if (local_socket_channel > 0) return 0;

    dund_log("Failed in accept local socket:%s\n", strerror(errno));
    return -1;
}

int close_local_channel(){
    if (local_socket_channel != -1) {
        close(local_socket_channel);
    }
    return 0;
}

void* local_channel_reader(void* argument){
    #define BUFFER_SIZE 1024
    char* buffer[BUFFER_SIZE];
    int length;
    while(1){
        if (local_socket_channel < 0) return (void*)-1;
        length = recv(local_socket_channel, buffer, BUFFER_SIZE, 0);
        //process buffer
        if (length < 0) {
            local_socket_channel = -1;
        } else {
            write_to_modem_channel(buffer, length);
        }
    }
}

int write_to_local_channel(char* buffer, size_t length){
    int remaining_length = length;
    int sent_length = 0;
    while(remaining_length > 0) {
        if (local_socket_channel < 0) return -1;
        sent_length = send(local_socket_channel, buffer + (length - remaining_length), remaining_length, 0);
        if (sent_length < 0) {
            local_socket_channel = -1;
        } else {
            remaining_length -= sent_length;
        }
    }
}

