/*
Copyright (c) 2009, Intel Corporation
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include <stdio.h>
#define _XOPEN_SOURCE
#define _GNU_SOURCE
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <fcntl.h>

#include "local_channel.h"
#include "dund_log.h"

extern int errno;
static int local_tty_channel = -1;

int open_local_channel(unsigned char* path){
    size_t local_size, remote_size;

   if ((local_tty_channel = getpt()) == -1) {  /* open tty for read */
       perror(strerror(errno));
       return -1;
   }

   if (grantpt(local_tty_channel) == -1) {  /* grant tty for read */
       perror(strerror(errno));
       return -1;
   }

   if (unlockpt(local_tty_channel) == -1) {  /* grant tty for read */
       perror(strerror(errno));
       return -1;
   }
   
   dund_log("slave tty:%s\n", ptsname(local_tty_channel));
   return 0;
}

int close_local_channel(){
    if (local_tty_channel != -1) {
        close(local_tty_channel);
    }
    return 0;
}

void* local_channel_reader(void* argument){
    #define BUFFER_SIZE 1024
    char buffer[BUFFER_SIZE];
    int length;
    while(1){
        if (local_tty_channel < 0) return -1;
        length = read(local_tty_channel, buffer, BUFFER_SIZE, 0);
        //process buffer
        if (length < 0) {
             if (errno != EIO) { 
                local_tty_channel = -1;
            } else {
//                printf("%s:%d\n", strerror(errno), __LINE__);
                  sleep(0);
            }
        } else {
            write_to_modem_channel(buffer, length);
        }
    }
    return 0;
}

int write_to_local_channel(char* buffer, size_t length){
    int remaining_length = length;
    int sent_length = 0;
    while(remaining_length > 0) {
        if (local_tty_channel < 0) return -1;
        sent_length = write(local_tty_channel, buffer + (length - remaining_length), remaining_length, 0);
        if (sent_length < 0) {
            if (errno != EIO) { 
                local_tty_channel = -1;
            } else {
                dund_log("%s\n", strerror(errno));
            }
        } else {
            remaining_length -= sent_length;
        }
    }
}

