/*
Copyright (c) 2009, Intel Corporation
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>

static char* local_channel = NULL;
static char* modem_channel = NULL;
static int is_daemon = 0;


static int print_usage(){
    printf("dund -d -l <local_channel_node> -m <modem_node>\n");
    return 0;
}

static int parse_arguments(int argc, char* argv[]) {
    int ch;
    while((ch = getopt(argc, argv, "dm:l:"))!= -1){
    switch(ch){
       case 'd':
          is_daemon = 1; break;
       case 'l':
          local_channel = optarg; break;
       case 'm':
          modem_channel = optarg; break;
     }
   }

   return 0;
}

void sig_handler(int signum) {
    shutdown_channels();
}

static int setup_sighandler(){
    struct sigaction act;

    act.sa_handler = sig_handler;
    act.sa_flags = 0;
    sigemptyset(&act.sa_mask);

    sigaction(SIGPIPE,  &act, 0);
    return 0;
}

int main(int argc, char* argv[])
{
    parse_arguments(argc, argv);

    if ((local_channel == NULL) || (modem_channel == NULL)) {
        print_usage();
        exit(-1);
    }

    setup_sighandler();

    if (is_daemon) {
        pid_t pid, sid;
        
        pid = fork();
    
        if (pid < 0) {
                exit(-1);
        }
    
        if (pid > 0) {
                exit(0);
        }
    
        umask(0);
                
        sid = setsid();
        if (sid < 0) {
                exit(-1);
        }
        
        if ((chdir("/")) < 0) {
                exit(-1);
        }
        
        close(STDIN_FILENO);
        close(STDOUT_FILENO);
        close(STDERR_FILENO);
    }

    while (1) {
        initialize_filter(local_channel, modem_channel);

        waitfor_filter_socket_close();
    }

    return 0;
}
