/*
Copyright (c) 2009, Intel Corporation
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#include <stddef.h>
#include <fcntl.h>
#include <sys/time.h>
#include <errno.h>
#include <assert.h>
#include <string.h>
#include <xlocale.h>
#include <errno.h>
#include <stdio.h>

#include "modem_channel.h"
#include "dund_log.h"

extern int errno;
static char modem_read_buffer[BUFFER_SIZE+21];
static char modem_write_buffer[BUFFER_SIZE+21];
static int buffer_write_pos = 0;
static struct timeval last_char_time = {0,0};

int current_modem_mode = COMMAND_MODE;
static int modem_fd = -1;
int at_command_echo = 1;
extern int errno;

static int dump_buffer(unsigned char* buffer, int length) {
    int i;
    char buf[4096];
    memset(buf, 0, 4096);
    snprintf(buf, length, "%s", buffer);
    dund_log("%s,", buf);
    for (i = 0; i < length; i ++) {
        dund_log("%02x ",buffer[i]);
        if (((i+1)%16) == 0) dund_log("|");
    }
    return 0;
}

int open_modem_channel(char* device_node, int option){
    modem_read_buffer[BUFFER_SIZE+20] = 0;
    modem_write_buffer[BUFFER_SIZE+20] = 0;
    current_modem_mode = COMMAND_MODE;
    return modem_fd = open(device_node, option);    
}

int close_modem_channel(){
    if (modem_fd != -1) close(modem_fd);
    return 0;
}

int switch_channle_mode(int fd, int next_mode){
    if (next_mode == current_modem_mode){
        //no change
        return 0;
    }
    
    //switch between Control &  DATA mode
    //follow the PPP sequence
    if (next_mode == DATA_MODE){
        
    } else if (next_mode == COMMAND_MODE){

    }
    current_modem_mode  = next_mode;
}

int write_to_modem_channel(char* buffer, size_t length){
    struct timeval now_time;
    int time_diff = 0;
    long time_diff_in_usec = 0;
    int i = 0;
    int escape_string_found = 0;
    int cached_plus_length = 0;

    if (length == 0 ) return 0;

    if (current_modem_mode == ESCAPING_DATA_MODE) return 0;

    if (current_modem_mode == DATA_MODE) {
        // if escape_string is found, drop the following data stream
        gettimeofday(&now_time, NULL);
    
        time_diff_in_usec = (now_time.tv_sec - last_char_time.tv_sec)* 1000 + \
                (now_time.tv_usec - last_char_time.tv_usec);

        if (time_diff_in_usec > INTERVAL_MILISECOND) {
            // as there is more than 1 second silence
            // need check if is escape string "+++"
            for (i = 0; i < 3; i++){
                if (i >= length) break;
                if (buffer[i] != '+') break;
                cached_plus_length ++;
            }

            if (i == 3)  {
                escape_string_found = 1;
                current_modem_mode = ESCAPING_DATA_MODE;
                cached_plus_length = 0;
            } else {
                // dont process the split string ("++" "+")
                cached_plus_length = 0;
            }
        }
        flush_buffer_to_modem(buffer, length);
   }

    if (current_modem_mode == COMMAND_MODE) {
        int last_pos = 0;
        if (at_command_echo == 1) {
            dund_log("Echo Command(%d): ", length);
            dump_buffer(buffer, length);
            dund_log("\n");

            write_to_local_channel(buffer, length);
        }

        //send line into the filter    
        for (i = 0; i < length; i ++){
            if (buffer[i] == '\r') {
                // line found
                if (buffer_write_pos + i > BUFFER_SIZE) {
			//buffer overflow, drop the buffer
		}
                memcpy((void*)(modem_write_buffer + buffer_write_pos), buffer + last_pos, (size_t)i - last_pos);
                modem_write_buffer[buffer_write_pos + i] = '\0';
                process_line(modem_write_buffer);
                buffer_write_pos = 0;
                last_pos = i + 1;
            }
        }
        //process the remain part
        memcpy((void*)(modem_write_buffer + buffer_write_pos), buffer + last_pos, length - last_pos);
        buffer_write_pos += length - last_pos;
    }
    return (modem_fd == -1) ? -1: 0;
}

int flush_buffer_to_modem(char* buffer, size_t length){
    int remain_length;
    int ret_value;
    remain_length = length; 
    while (remain_length > 0) {
        if (modem_fd == -1) return -1;
        ret_value = write(modem_fd, buffer + (length - remain_length), remain_length);
        if (ret_value > 0) {
            remain_length -= ret_value;
        } else {
            current_modem_mode = MODEM_FD_ERROR;
            dund_log("change modem_fd:%d:%d(%s)\n", modem_fd, __LINE__, strerror(errno));
            modem_fd = -1;
            break;
        }
    }

    if ((modem_fd != -1) && (length > 0)) {
       dund_log("PPP --> Modem(%d):", length);
       dump_buffer(buffer, length);
       dund_log("\n");
    }
}

static int process_command_input(int offset, size_t length)
{
    #undef MAGIC_STRING
    #define MAGIC_STRING "\r\nCONNECT" 

    char* pos = NULL;
    //GUARD check
    assert(modem_read_buffer[BUFFER_SIZE+20] == 0); 

    dund_log("processing command buffer:%s\n", modem_read_buffer);
    pos = strstr(modem_read_buffer, MAGIC_STRING);
    if (pos){
        // found
        current_modem_mode = DATA_MODE;
    } else {
        // dont worry about the over flow
        memcpy( modem_read_buffer, modem_read_buffer + (offset + length - 20), 20);
    }
}

static int process_data_in_input(int offset, size_t length)
{
    #undef MAGIC_STRING
    #define MAGIC_STRING "\r\nOK\r\n"
    int pos =0;
    assert(length >= strlen(MAGIC_STRING));

    if (strncpy(modem_read_buffer + offset, MAGIC_STRING, strlen(MAGIC_STRING) == 0)) {
        current_modem_mode = DATA_MODE;
    } else {
        //should be something wrong, waiting for next escape string
        current_modem_mode = COMMAND_MODE;
    }
}

static int process_data_out_input(int offset, size_t length)
{
    // in escaping mode, monitor "\r\nOK\r\n"
    #undef MAGIC_STRING
    #define MAGIC_STRING "\r\nOK\r\n"
    int pos =0;

    //GUARD check
    assert(length >= strlen(MAGIC_STRING));

    if (strncpy(modem_read_buffer + offset, MAGIC_STRING, strlen(MAGIC_STRING) == 0)) {
        current_modem_mode = COMMAND_MODE;
    } else {
        //should be something wrong, waiting for next escape string
        current_modem_mode = DATA_MODE;
    }
}

static int process_data_input(int offset, size_t length)
{
    // in escaping mode, monitor "\r\nNO CARRIER\r\n"
    // as ppp will transform "\0", so we can still using strcasstr to search the string
    #undef MAGIC_STRING
    #define MAGIC_STRING "\r\nNO CARRIER\r\n"
    
    assert(modem_read_buffer[BUFFER_SIZE+20] == 0); 
    char *pos = NULL;
    pos = strstr(modem_read_buffer, MAGIC_STRING);
    if (pos){
        // found
        current_modem_mode = COMMAND_MODE;
    } else {
        // dont worry about the over flow
        memcpy( modem_read_buffer, modem_read_buffer + (offset + length - 20), 20);
    }
}

int read_from_modem(){
    // In command mode, monitor "\r\nCONNECT\r\n"
    // in entering mode, monitor "OK\r"
    // in data mode, monitor "\r\nNO CARRIER\r\n"
    // the basic algorithm is 
    // 1. to search the string in the incoming buffer
    // 2. if no match found, cache the enough buffer for next searching
    int ret_value = 0;
    char *buffer = modem_read_buffer + 20;

    ret_value = read(modem_fd, buffer, BUFFER_SIZE);
    if (ret_value < 0) {
        dund_log(strerror(errno));
        current_modem_mode = MODEM_FD_ERROR;
        dund_log("change modem_fd:%d:%d\n",modem_fd, __LINE__);
        modem_fd = -1;
    }

    if (current_modem_mode == MODEM_FD_ERROR) {
        dund_log("Bad modem FD:%d(%s)\n", modem_fd, strerror(errno));
        return -1;
    }

    if (ret_value > 0) {
    dund_log("Modem --> PPP(%d): ", ret_value);
    dump_buffer(buffer, ret_value);
    dund_log("\n");
    }

    dund_log("process mode input:%d\n", current_modem_mode);

    switch(current_modem_mode) {
    case COMMAND_MODE:
        process_command_input(20, ret_value); break;
    case DATA_MODE:
        process_data_input(20, ret_value); break;
    case ENTERING_DATA_MODE:
        process_data_in_input(20, ret_value); break;
    case ESCAPING_DATA_MODE:
        process_data_out_input(20, ret_value); break;
    default:
        //Error
        dund_log(strerror(errno));
    }

    dund_log("process mode input:%d\n", current_modem_mode);

    //flush buffer into local channel
    if (ret_value > 0) {
        dund_log("Modem --> PPP(%d): ", ret_value);
        dump_buffer(buffer, ret_value);
        dund_log("\n");
        write_to_local_channel(buffer, ret_value);
    }
    return 0;
}

void * modem_reader(void * argument) {
    //initialize modem
    flush_buffer_to_modem("ATE0\r\n", strlen("ATE0\r\n"));
    read(modem_fd, modem_read_buffer, BUFFER_SIZE);
    while(1) {
        if (read_from_modem() < 0) break;
    }
    return NULL;
}
